// Code to run the halloween dragon
// By Graham and Jason Armitage
// 23 Oct 2023
// Include the libraries:
#include <Wire.h>
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
#include <Servo.h>

Servo neckServo;  // create servo object to control a servo
Servo headServo;
Servo jawServo;
Servo flameServo;

// Use pins 10 and 11 to communicate with DFPlayer Mini
static const uint8_t PIN_MP3_TX = 11; // Connects to module's RX 
static const uint8_t PIN_MP3_RX = 10; // Connects to module's TX 
SoftwareSerial softwareSerial(PIN_MP3_RX, PIN_MP3_TX);
// Create the Player object
DFRobotDFPlayerMini player;

int headServoPin = 9;
int neckServoPin = 8;
int jawServoPin = 7;
int flameServoPin = 6;
int sparkPin = 5;
int busyPin = 2; 

int dur ;  // duration of move in millis
int dir = 1; // Direction of movement 0=left, 1=right
int headPos = 90; // last position angle of the head
int neckPos = 90; // last position angle of the neck
int endPos; // position angle to move to
long range = 60;  // max movement range of head and neck
int count = 0;

unsigned long MOVING_TIME = 3000; // moving time is 3 seconds
unsigned long moveStartTime;

// #####################################################################
void setup() {
  // Begin serial communication at a baud rate of 9600:
  Serial.begin(9600);
  // Init serial port for DFPlayer Mini
  softwareSerial.begin(9600);
  // Start communication with DFPlayer Mini
  if (player.begin(softwareSerial)) {
    Serial.println("DFPPlayer initialized");
    player.volume(30);
    player.disableLoop();
    playMP3(2);

  } else {
    Serial.println("Connecting to DFPlayer Mini failed!");
  }

  neckServo.attach(neckServoPin);  // attaches the servo on pin 9 to the servo object
  headServo.attach(headServoPin);  // attaches the servo on pin 8 to the servo object
  jawServo.attach(jawServoPin);  // attaches the servo on pin 7 to the servo object
  flameServo.attach(flameServoPin);  // attaches the servo on pin 6 to the servo object
  pinMode(sparkPin,OUTPUT);
  pinMode(busyPin, INPUT);

  // postion head and neck in center
  headServo.write(headPos);
  neckServo.write(headPos);
  flameServo.write(0);
  jawServo.write(0);
  digitalWrite(sparkPin, LOW);

  randomSeed(42);
}
// #####################################################################
void playMP3(int track) {
    // only play sound of not already playing something
    if (digitalRead(busyPin)) { // High if track has stopped playing
      player.play(track);
    
    }
}
// #####################################################################
void moveNeck(long startAngle, long stopAngle, long dur) {
  unsigned long MOVING_TIME = dur;  // pass in duration of move
  unsigned long moveStartTime = millis(); // start moving
  unsigned long progress = millis() - moveStartTime;
  int angle;
  while (progress <= MOVING_TIME) {
    angle = map(progress, 0, MOVING_TIME, startAngle, stopAngle);
    neckServo.write(angle);
    delay(20);
    progress = millis() - moveStartTime;
  }
}
// #####################################################################
void moveHead(long startAngle, long stopAngle, long dur) {
  unsigned long MOVING_TIME = dur;  // pass in duration of move
  unsigned long moveStartTime = millis(); // start moving
  unsigned long progress = millis() - moveStartTime;
  int angle;
  while (progress <= MOVING_TIME) {
    angle = map(progress, 0, MOVING_TIME, startAngle, stopAngle);
    headServo.write(angle);
    delay(20);
    progress = millis() - moveStartTime;
  }
}
// #####################################################################
void fireFlame() {
    // Squirt the gas
    flameServo.write(40);
    delay(100);
        // start the igniter
    digitalWrite(sparkPin, HIGH);
        delay(200);
    // turn off spark
    digitalWrite(sparkPin, LOW);
    //stop the gas
    flameServo.write(0);  // return to zero
    delay(100);
}

// #####################################################################
void loop() {
  count++;

  // randomly move the neck and head
  // intermittently stop, open jaw and shoot flame
  // head moves first the neck follows
  // randomly decide left or right, and how far to move
  // small head movement = small neck movement and vice versa

  // randomly play a sound
  int x = random(12);
  if (x < 4) {
    playMP3(x+1);
  }

  dir = -dir;  // swap direction each time; 
  moveStartTime = millis();
  if (dir < 0) {  // go left
    endPos = int(random(100,140));  // choose a spot between 100 and 140 deg, center to left
    moveHead(headPos, endPos, 2000);
    headPos = endPos;  // update the last position

    if (headPos > 100) {
      endPos = 120;  // move neck to max side movement
      moveNeck(neckPos, endPos, 2000);
      neckPos = endPos;  // update the last neck position
    }

  } else {  // go right
    //Serial.println("move Right");
    endPos = int(random(30,85));  // choose a spot between 30 and 85 deg, right to center
    moveHead(headPos, endPos, 2000);
    headPos = endPos;

    if (headPos < 60) {
      endPos = 50;  // move half the angle the head moves
      moveNeck(neckPos, endPos, 2000);
      neckPos = endPos;  // update the last neck position
    }
    
  }

  delay(random(2000));  // add in a pause

  //randomly open jaw and shoot flames out
  if (random(20)<2) {
    fireFlame();
  }
  delay(500);
}
